# Stuvus-Bot

A gitlab bot based on gidgetlab for helping with various small tasks in gitlab repos.

## Running

To run this bot, you need to have a world-reachable host.

The prefered way of running this bot is using docker:

```bash
docker run -p <some_port>:8080 -e "OPTION1=foo" -e "OPTION2=bar" cr.stuvus.de/ref-it/stuvus-bot
```

It can also be run from the command line using python, requiring [poetry](https://python-poetry.org):

```bash
#Install dependencies
poetry install --no-dev

#Run bot
export OPTION1=foo; export OPTION2=bar; poetry run stuvus-bot
```

The Options are explained at [Configuration](#configuration).
In both cases you probably want to deploy some reverse proxy infront of the bot for SSL/TLS terminiation.

Now go to your gitlab project settings and select Webhooks to create a webhook:

- In the URL field, enter the URL under which the bot is reachable.
- In the Secret Token field, enter the `GL_SECRET` String configured for the bot (see [Configuration](#configuration)
- Under Trigger, select Comments and Merge request events
- Leave Enable SSL verification enabled (unless you didn't configure SSL/TLS with a reverse proxy)
- Click Add webhook

## Configuration

All configuration is performed using environment variables:

| Environment Variable | Required/Default | Description |
| --- | --- | --- |
| GL_URL | required | The URL of your gitlab server, e.g. `git.example.com`
| GL_USER | required | The user with which to connect to the gitlab server
| GL_ACCESS_TOKEN | required | A valid access token for that user having the `api` scope.
| GL_SECRET | required | An arbitrary string used to authenticate incoming webhook requests. Also needs to be configured in the gitlab webhook settings. (See [Running](#running))

## Development

When adding features to this bot, add each feature in a seperate python file in the `features` module.

The file should start with:

```python
from gidgetlab.routing import Router

router_<feature_name> = Router()
```

and contain one or more event handlers like this:

```python
@router_<feature_name>.register("Issue Hook", action="open")
async def issue_opened_event(event: Event, gl: GitLabAPI, *args, **kwargs:
    ...
```

Finally register the new feature by importing the `router_<feature_name>` in `main.py` and adding it to the list of registered routers.
